#!/bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Set Stub Transfer Service
#   Utility script to set all the transfer agents on a machine to use the stub transfer service. 
#
# Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
# Version info: $Id: set_stub_transfer_service.sh,v 1.2 2008/06/11 13:39:44 ptedesco Exp $
# Release: $Name:  $
#

# Variables
#-------------------------------------------------------------------------------
# name of the script
SCRIPT_NAME=$(basename ${0})
# temporary file names for the xslt files
XSLT_TRANSFORM="/tmp/$(uuidgen).xslt"
XSLT_CHECK_URLCOPY="/tmp/$(uuidgen).xslt"
XSLT_CHECK_STUB="/tmp/$(uuidgen).xslt"
# the directory where the configuration files are in
CONFIG_DIR_DEF="/opt/glite/etc/glite-data-transfer-agents.d"
CONFIG_DIR=${CONFIG_DIR_DEF}

# Command line parsing
#-------------------------------------------------------------------------------
# show script usage
function ShowHelp(){
    echo
    echo "Usage : ${SCRIPT_NAME} [OPTIONS]"
    echo
    echo "Create a backup copy of transfer agents configuration files and configure"
    echo "the agents to use the stub transfer service."
    echo
    echo " -d <dir>:   configuration files directory (default = ${CONFIG_DIR_DEF})"
    echo " -h:         print this help message and exit"
}

# process command line parameters
while getopts ":d:h" option
do
    case ${option} in
    d )
        CONFIG_DIR=${OPTARG}
        if [ ! -d ${CONFIG_DIR} ]; then
            echo "Error: ${CONFIG_DIR} is not a valid directory."
            exit 1
        fi
        ;;
    h )
        ShowHelp
        exit 0
        ;;
    * )
        echo "invalid options"
        ShowHelp
        exit 1
        ;;
    esac
done
shift $(($OPTIND - 1))

# print execution parameters
echo "Configuration files directory: ${CONFIG_DIR}"

# xslt files
#-------------------------------------------------------------------------------
cat > ${XSLT_TRANSFORM} << EOD
<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" encoding="UTF-8" indent="yes"/>
    <xsl:strip-space elements="*"/>
    <!-- identity transform -->
    <xsl:template match="@*|node()">
        <xsl:copy>
            <xsl:apply-templates select="@*|node()" />
        </xsl:copy>
    </xsl:template>
    <!-- replace the transfer service node -->
    <xsl:template match="/service/components/component">
        <xsl:choose>
            <xsl:when test="@name='transfer-agent-ts-urlcopy'">
                <component name="transfer-agent-ts-stub">
                    <lib>libglite_data_transfer_agent_ts_stub.so</lib>
                    <init>
                        <param name="MaxTransfers">
                            <value>100</value>
                        </param>
                        <param name="MaxRequestSize">
                            <value>10</value>
                        </param>
                    </init>
                    <config/>
                </component>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy>
                    <xsl:apply-templates select="@*|node()" />
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>
EOD

cat > ${XSLT_CHECK_URLCOPY} << EOD
<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="text" encoding="UTF-8" indent="no"/>
    <xsl:preserve-space elements="*"/>
    <xsl:template match="/">
        <xsl:for-each select="service/components/component[@name='transfer-agent-ts-urlcopy']">Yes</xsl:for-each>
    </xsl:template>
</xsl:stylesheet>
EOD

cat > ${XSLT_CHECK_STUB} << EOD
<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="text" encoding="UTF-8" indent="no"/>
    <xsl:preserve-space elements="*"/>
    <xsl:template match="/">
        <xsl:for-each select="service/components/component[@name='transfer-agent-ts-stub']">Yes</xsl:for-each>
    </xsl:template>
</xsl:stylesheet>
EOD

# check if given configuration file is using the urlcopy transfer service
function check_urlcopy(){
    local CHECK=$(xsltproc ${XSLT_CHECK_URLCOPY} ${1})
    if [ "Yes" == "${CHECK}" ]; then
        return 0
    else
        return 1
    fi
}
# check if given configuration file is using the stub transfer service
function check_stub(){
    local CHECK=$(xsltproc ${XSLT_CHECK_STUB} ${1})
    if [ "Yes" == "${CHECK}" ]; then
        return 0
    else
        return 1
    fi
}

# Actual execution
#-------------------------------------------------------------------------------
# make a copy of configuration files
IFS=$'\n'
CONFIGURATION_FILES=$(find ${CONFIG_DIR} -type f -name "glite-transfer-channel-agent-*.properties.xml" -maxdepth 1)
unset FILES_TO_PATCH
unset FILES_TO_PATCH[*]
echo
echo "Creating backup copies of configuration files..."
for FILE in ${CONFIGURATION_FILES}; do 
    check_urlcopy ${FILE} && {
        echo "Creating backup of ${FILE}"
        cp "${FILE}" "${FILE}.backup"
        FILES_TO_PATCH[${#FILES_TO_PATCH[*]}]="${FILE}"
    } || {
        echo "Warning: $(basename ${FILE}) does not appear to be configured to use transfer-url-copy transfer service. Skipping."
    }
done

# apply transformation
echo
echo "Configuring agents to use stub transfer service..."
for ((i=0;i<${#FILES_TO_PATCH[*]};i++)); do
    FILE=${FILES_TO_PATCH[${i}]}
    echo "processing ${FILE}"
    xsltproc ${XSLT_TRANSFORM} ${FILE}.backup > ${FILE}
done
echo
echo "${#FILES_TO_PATCH[*]} agents configured to use stub transfer service."

# cleanup
#-------------------------------------------------------------------------------
rm ${XSLT_TRANSFORM}
rm ${XSLT_CHECK_URLCOPY}
rm ${XSLT_CHECK_STUB}

